import java.util.Random;

public class ListInsertionTester {
    public static final void main(String[] args) {

	// instance sizes we'll test
	int[] sizes = getSizeArray(50000, 1000);

	// the number of operations to do for each instance size
	int numOps = 1000;

	SimpleList<Integer> arraySLOne;
	SimpleList<Integer> linkedSL;

	String[] dataLabels = {"instance size", "LSL rand add time (ns)", "ASL rand add time (ns)", "LSL front add time (ns)", "ASL front add time (ns)", "LSL back add time (ns)", "ASL back add time (ns)"};
	int[][] data = new int[dataLabels.length][];
	data[0] = sizes;

	arraySLOne = new ArraySimpleListOne<Integer>();
	linkedSL = new LinkedSimpleList<Integer>();

	data[1] = testAddRandom(linkedSL, sizes, numOps);
	data[2] = testAddRandom(arraySLOne, sizes, numOps);

	arraySLOne = new ArraySimpleListOne<Integer>();
	linkedSL = new LinkedSimpleList<Integer>();

	data[3] = testAddFront(linkedSL, sizes, numOps);
	data[4] = testAddFront(arraySLOne, sizes, numOps);

	arraySLOne = new ArraySimpleListOne<Integer>();
	linkedSL = new LinkedSimpleList<Integer>();
	
	data[5] = testAddBack(linkedSL, sizes, numOps);
	data[6] = testAddBack(arraySLOne, sizes, numOps);
	

	writeData("simple-list-insert-test.csv", dataLabels, data);
    }
    
    static void initializeList(SimpleList<Integer> list, int size) {
	while (list.size() < size) {
	    list.add(0, 0);
	}

	while (list.size() > size) {
	    list.remove(0);
	}
    }
    
    static int[] testAddRandom(SimpleList<Integer> list,
				int[] sizes,
				int numOps) {
	
	RunTimer rt = new RunTimer();
	Random rand = new Random(42);

	int[] runtimes = new int[sizes.length];

	for (int i = 0; i < sizes.length; ++i) {

	    int size = sizes[i];
	    
	    initializeList(list, size);
	    rt.reset();

	    for (int j = 0; j < numOps; ++j) {
		rt.start();
		list.add(rand.nextInt(list.size()), 0);
		rt.stop();

		list.remove(0);
	    }

	    runtimes[i] = (int) (rt.getElapsedNanos() / numOps); 

	}

	return runtimes;
	
    }

    static int[] testAddFront(SimpleList<Integer> list,
				int[] sizes,
				int numOps) {
	
	RunTimer rt = new RunTimer();

	int[] runtimes = new int[sizes.length];

	for (int i = 0; i < sizes.length; ++i) {

	    int size = sizes[i];
	    
	    initializeList(list, size);
	    rt.reset();

	    for (int j = 0; j < numOps; ++j) {
		rt.start();
		list.add(0, 0);
		rt.stop();

		list.remove(0);
	    }

	    runtimes[i] = (int) (rt.getElapsedNanos() / numOps); 

	}

	return runtimes;
	
    }

    static int[] testAddBack(SimpleList<Integer> list,
				int[] sizes,
				int numOps) {
	
	RunTimer rt = new RunTimer();

	int[] runtimes = new int[sizes.length];

	for (int i = 0; i < sizes.length; ++i) {

	    int size = sizes[i];
	    
	    initializeList(list, size);
	    rt.reset();

	    for (int j = 0; j < numOps; ++j) {
		rt.start();
		list.add(list.size(), 0);
		rt.stop();

		list.remove(0);
	    }

	    runtimes[i] = (int) (rt.getElapsedNanos() / numOps); 

	}

	return runtimes;
	
    }
    
    

    static void writeData(String path, String[] labels, int[][] data) {
	CSVWriter csv = new CSVWriter(path);

	for (String label : labels) {
	    csv.addEntry(label);
	}

	csv.endLine();

	for (int i = 0; i < data[0].length; ++i) {
	    for (int j = 0; j < data.length; ++j) {
		csv.addEntry(data[j][i]);
	    }

	    csv.endLine();
	}

	csv.close();
    }
    

    static int[] getSizeArray(int max, int step) {
	int[] sizes = new int[max / step];
	int size = step;

	for (int i = 0; i < max / step; ++i) {
	    sizes[i] = step + i * step;
	}

	return sizes;
    }
    
}
